@@ -1,6 +1,6 @@
 package RT::Extension::LDAPImport;
 
-our $VERSION = '0.34';
+our $VERSION = '0.35';
 
 use warnings;
 use strict;
@@ -52,6 +52,38 @@ Running the import:
     /opt/rt4/local/plugins/RT-Extension-LDAPImport/bin/rtldapimport \
     --import
 
+=head1 INSTALLATION
+
+=over
+
+=item C<perl Makefile.PL>
+
+=item C<make>
+
+=item C<make install>
+
+May need root permissions
+
+=item Edit your F</opt/rt4/etc/RT_SiteConfig.pm>
+
+If you are using RT 4.2 or greater, add this line:
+
+    Plugin('RT::Extension::LDAPImport');
+
+For earlier releases of RT 4, add this line:
+
+    Set(@Plugins, qw(RT::Extension::LDAPImport));
+
+or add C<RT::Extension::LDAPImport> to your existing C<@Plugins> line.
+
+=item Clear your mason cache
+
+    rm -rf /opt/rt4/var/mason_data/obj
+
+=item Restart your webserver
+
+=back
+
 =head1 CONFIGURATION
 
 All of the configuration for the importer goes
@@ -89,17 +121,77 @@ The LDAP search filter to apply (in this case, find all the users).
                        WorkPhone    => 'telephoneNumber',
                        Organization => 'departmentName'});
 
-This provides the mapping of attributes in RT to attribute in LDAP.
+This provides the mapping of attributes in RT to attribute(s) in LDAP.
 Only Name is required for RT.
 
-The LDAP attributes can also be an arrayref of LDAP fields
+The values in the mapping (i.e. the LDAP fields, the right hand side)
+can be one of the following:
+
+=over 4
+
+=item an attribute
+
+LDAP attribute to use. Only first value is used if attribute is
+multivalue. For example:
+
+    EmailAddress => 'mail',
+
+=item an array reference
+
+The LDAP attributes can also be an arrayref of LDAP fields,
+for example:
 
     WorkPhone => [qw/CompanyPhone Extension/]
 
-which will be concatenated together with a space.
+which will be concatenated together with a space. First values
+of each attribute are used in case they have multiple values.
 
-The LDAP attribute can also be a subroutine reference
-that returns either an arrayref or a list of attributes.
+=item a subroutine reference
+
+The LDAP attribute can also be a subroutine reference that does
+mapping, for example:
+
+    YYY => sub {
+        my %args = @_;
+        my @values = grep defined && length, $args{ldap_entry}->get_value('XXX');
+        return @values;
+    },
+
+The subroutine should return value or list of values. The following
+arguments are passed into the function in a hash:
+
+=over 4
+
+=item self
+
+Instance of this class.
+
+=item ldap_entry
+
+L<Net::LDAP::Entry> instance that is currently mapped.
+
+=item import
+
+Boolean value indicating whether it's import or a dry run. If it's
+dry run (import is false) then function shouldn't change anything.
+
+=item mapping
+
+Hash reference with the currently processed mapping, eg. C<$LDAPMapping>.
+
+=item rt_field and ldap_field
+
+The currently processed key and value from the mapping.
+
+=item result
+
+Hash reference with results of completed mappings for this ldap entry.
+This should be used to inject that are not in the mapping, not to inspect.
+Mapping is processed in literal order of the keys.
+
+=back
+
+=back
 
 The keys in the mapping (i.e. the RT fields, the left hand side) may be a user
 custom field name prefixed with C<UserCF.>, for example C<< 'UserCF.Employee
@@ -161,12 +253,16 @@ The search filter to apply.
 
 A mapping of RT attributes to LDAP attributes to identify group members.
 Name will become the name of the group in RT, in this case pulling
-from the cn attribute on the LDAP group record returned.
+from the cn attribute on the LDAP group record returned. Everything
+besides C<Member_Attr_Value> is processed according to rules described
+in documentation for C<$LDAPMapping> option, so value can be array
+or code reference besides scalar.
 
 C<Member_Attr> is the field in the LDAP group record the importer should
 look at for group members. These values (there may be multiple members)
 will then be compared to the RT user name, which came from the LDAP
-user record.
+user record. See F<t/group-callbacks.t> for a complex example of
+using a code reference as value of this option.
 
 C<Member_Attr_Value>, which defaults to 'dn', specifies where on the LDAP
 user record the importer should look to compare the member value.
@@ -174,6 +270,13 @@ A match between the member field on the group record and this
 identifier (dn or other LDAP field) on a user record means the
 user will be added to that group in RT.
 
+C<id> is the field in LDAP group record that uniquely identifies
+the group. This is optional and shouldn't be equal to mapping for
+Name field. Group names in RT must be distinct and you don't need
+another unique identifier in common situation. However, when you
+rename a group in LDAP, without this option set properly you end
+up with two groups in RT.
+
 You can provide a C<Description> key which will be added as the group
 description in RT. The default description is 'Imported from LDAP'.
 
@@ -238,6 +341,15 @@ It may work with RT 3.6.
 The L<ldapsearch|http://www.openldap.org/software/man.cgi?query=ldapsearch&manpath=OpenLDAP+2.0-Release>
 utility in openldap can be very helpful while refining your filters.
 
+=head1 Developing
+
+If you want to run tests for this extension, you should create the
+F<inc/.author> directory and will need to set RT_DBA_USER and
+RT_DBA_PASSWORD environment variables to a database user that can
+create/drop tests databases as needed.
+
+Do not run tests in a production environment.
+
 =head1 METHODS
 
 =head2 connect_ldap
@@ -429,14 +541,6 @@ sub import_users {
     my $done = 0; my $count = scalar @results;
     while (my $entry = shift @results) {
         my $user = $self->_build_user_object( ldap_entry => $entry );
-        unless ( $user->{Name} ) {
-            $self->_warn("No Name or Emailaddress for user, skipping ".Dumper $user);
-            next;
-        }
-        if ( $user->{Name} =~ /^[0-9]+$/) {
-            $self->_debug("Skipping user '$user->{Name}', as it is numeric");
-            next;
-        }
         $self->_import_user( user => $user, ldap_entry => $entry, import => $args{import} );
         $done++;
         $self->_debug("Imported $done/$count users");
@@ -446,7 +550,8 @@ sub import_users {
 
 =head2 _import_user
 
-The user has run us with --import, so bring data in.
+We have found a user to attempt to import; returns the L<RT::User>
+object if it was found (or created), C<undef> if not.
 
 =cut
 
@@ -454,6 +559,15 @@ sub _import_user {
     my $self = shift;
     my %args = @_;
 
+    unless ( $args{user}{Name} ) {
+        $self->_warn("No Name or Emailaddress for user, skipping ".Dumper($args{user}));
+        return;
+    }
+    if ( $args{user}{Name} =~ /^[0-9]+$/) {
+        $self->_debug("Skipping user '$args{user}{Name}', as it is numeric");
+        return;
+    }
+
     $self->_debug("Processing user $args{user}{Name}");
     $self->_cache_user( %args );
 
@@ -464,7 +578,7 @@ sub _import_user {
     $self->add_custom_field_value( %args );
     $self->update_object_custom_field_values( %args, object => $args{user} );
 
-    return 1;
+    return $args{user};
 }
 
 =head2 _cache_user ldap_entry => Net::LDAP::Entry, [user => { ... }]
@@ -568,72 +682,132 @@ sub _build_user_object {
 
 =head2 _build_object
 
-Builds up data from LDAP for importing
-Returns a hash of user or group data ready for
-C<RT::User::Create> or C<RT::Group::Create>.
+Internal method - a wrapper around L</_parse_ldap_mapping>
+that flattens results turning every value into a scalar.
+
+The following:
+
+    [
+        [$first_value1, ... ],
+        [$first_value2],
+        $scalar_value,
+    ]
+
+Turns into:
+
+    "$first_value1 $first_value2 $scalar_value"
+
+Arguments are just passed into L</_parse_ldap_mapping>.
 
 =cut
 
 sub _build_object {
     my $self = shift;
     my %args = @_;
-    my $mapping = $args{mapping};
 
-    my $object = {};
-    foreach my $rtfield ( keys %{$mapping} ) {
-        next if $rtfield =~ $args{skip};
-        my $ldap_attribute = $mapping->{$rtfield};
-
-        my @attributes = $self->_parse_ldap_mapping($ldap_attribute);
-        unless (@attributes) {
-            $self->_error("Invalid LDAP mapping for $rtfield ".Dumper($ldap_attribute));
-            next;
-        }
-        my @values;
-        foreach my $attribute (@attributes) {
-            #$self->_debug("fetching value for $attribute and storing it in $rtfield");
-            # otherwise we'll pull 7 alternate names out of the Name field
-            # this may want to be configurable
-            push @values, scalar $args{ldap_entry}->get_value($attribute);
-        }
-        $object->{$rtfield} = join(' ',grep {defined} @values);
+    my $res = $self->_parse_ldap_mapping( %args );
+    foreach my $value ( values %$res ) {
+        @$value = map { ref $_ eq 'ARRAY'? $_->[0] : $_ } @$value;
+        $value = join ' ', grep defined && length, @$value;
     }
-
-    return $object;
+    return $res;
 }
 
 =head3 _parse_ldap_mapping
 
-Internal helper function for C<import_user>.
-If we're passed an arrayref, it will recurse
-over each of the elements in case one of them is
-another arrayref or subroutine.
+Internal helper method that maps an LDAP entry to a hash
+according to passed arguments. Takes named arguments:
 
-If we're passed a subref, it executes the code
-and recurses over each of the returned values
-so that a returned array or arrayref will work.
+=over 4
 
-If we're passed a scalar, returns that.
+=item ldap_entry
 
-Returns a list of values that need to be concatenated
-together.
+L<Net::LDAP::Entry> instance that should be mapped.
+
+=item only
+
+Optional regular expression. If passed then only matching
+entries in the mapping will be processed.
+
+=item only
+
+Optional regular expression. If passed then matching
+entries in the mapping will be skipped.
+
+=item mapping
+
+Hash that defines how to map. Key defines position
+in the result. Value can be one of the following:
+
+If we're passed a scalar or an array reference then
+value is:
+
+    [
+        [value1_of_attr1, value2_of_attr1],
+        [value1_of_attr2, value2_of_attr2],
+    ]
+
+If we're passed a subroutine reference as value or
+as an element of array, it executes the code
+and returned list is pushed into results array:
+
+    [
+        @result_of_function,
+    ]
+
+All arguments are passed into the subroutine as well
+as a few more. See more in description of C<$LDAPMapping>
+option.
+
+=back
+
+Returns hash reference with results, each value is
+an array with elements either scalars or arrays as
+described above.
 
 =cut
 
 sub _parse_ldap_mapping {
     my $self = shift;
-    my $mapping = shift;
+    my %args = @_;
 
-    if (ref $mapping eq 'ARRAY') {
-        return map { $self->_parse_ldap_mapping($_) } @$mapping;
-    } elsif (ref $mapping eq 'CODE') {
-        return map { $self->_parse_ldap_mapping($_) } $mapping->()
-    } elsif (ref $mapping) {
-        $self->_error("Invalid type of LDAPMapping [$mapping]");
-        return;
-    } else {
-        return $mapping;
+    my $mapping = $args{mapping};
+
+    my %res;
+    foreach my $rtfield ( sort keys %$mapping ) {
+        next if $args{'skip'} && $rtfield =~ $args{'skip'};
+        next if $args{'only'} && $rtfield !~ $args{'only'};
+
+        my $ldap_field = $mapping->{$rtfield};
+        my @list = grep defined && length, ref $ldap_field eq 'ARRAY'? @$ldap_field : ($ldap_field);
+        unless (@list) {
+            $self->_error("Invalid LDAP mapping for $rtfield, no defined fields");
+            next;
+        }
+
+        my @values;
+        foreach my $e (@list) {
+            if (ref $e eq 'CODE') {
+                push @values, $e->(
+                    %args,
+                    self => $self,
+                    rt_field => $rtfield,
+                    ldap_field => $ldap_field,
+                    result => \%res,
+                );
+            } elsif (ref $e) {
+                $self->_error("Invalid type of LDAP mapping for $rtfield, value is $e");
+                next;
+            } else {
+                # XXX: get_value asref returns undef if there is no such field on
+                # the entry, should we warn?
+                push @values, grep defined, $args{'ldap_entry'}->get_value( $e, asref => 1 );
+            }
+        }
+        $res{ $rtfield } = \@values;
     }
+
+    return \%res;
 }
 
 =head2 create_rt_user
@@ -792,25 +966,18 @@ sub add_custom_field_value {
     my %args = @_;
     my $user = $args{user};
 
-    foreach my $rtfield ( keys %{$RT::LDAPMapping} ) {
+    my $data = $self->_build_object(
+        %args,
+        only => qr/^CF\.(.+)$/i,
+        mapping => $RT::LDAPMapping,
+    );
+
+    foreach my $rtfield ( keys %$data ) {
         next unless $rtfield =~ /^CF\.(.+)$/i;
         my $cf_name = $1;
-        my $ldap_attribute = $RT::LDAPMapping->{$rtfield};
 
-        my @attributes = $self->_parse_ldap_mapping($ldap_attribute);
-        unless (@attributes) {
-            $self->_error("Invalid LDAP mapping for $rtfield ".Dumper($ldap_attribute));
-            next;
-        }
-        my @values;
-        foreach my $attribute (@attributes) {
-            #$self->_debug("fetching value for $attribute and storing it in $rtfield");
-            # otherwise we'll pull 7 alternate names out of the Name field
-            # this may want to be configurable
-            push @values, scalar $args{ldap_entry}->get_value($attribute);
-        }
-        my $cfv_name = join(' ',@values); 
-        next unless $cfv_name;
+        my $cfv_name = $data->{ $rtfield }
+            or next;
 
         my $cf = RT::CustomField->new($RT::SystemUser);
         my ($status, $msg) = $cf->Load($cf_name);
@@ -859,21 +1026,18 @@ sub update_object_custom_field_values {
     my %args = @_;
     my $obj  = $args{object};
 
-    foreach my $rtfield ( keys %{$RT::LDAPMapping} ) {
+    my $data = $self->_build_object(
+        %args,
+        only => qr/^UserCF\.(.+)$/i,
+        mapping => $RT::LDAPMapping,
+    );
+
+    foreach my $rtfield ( keys %$data ) {
         # XXX TODO: accept GroupCF when we call this from group_import too
         next unless $rtfield =~ /^UserCF\.(.+)$/i;
         my $cf_name = $1;
-        my $ldap_attribute = $RT::LDAPMapping->{$rtfield};
-
-        my @attributes = $self->_parse_ldap_mapping($ldap_attribute);
-        unless (@attributes) {
-            $self->_error("Invalid LDAP mapping for $rtfield ".Dumper($ldap_attribute));
-            next;
-        }
-        my $value = join ' ',
-                    grep { defined and length }
-                     map { scalar $args{ldap_entry}->get_value($_) }
-                         @attributes;
+        # XXX TODO: value can not be undefined, but empty string
+        my $value = $data->{$rtfield};
 
         my $current = $obj->FirstCustomFieldValue($cf_name);
 
@@ -926,7 +1090,18 @@ sub import_groups {
 
     my $done = 0; my $count = scalar @results;
     while (my $entry = shift @results) {
-        my $group = $self->_build_object( ldap_entry => $entry, skip => qr/(?i)^Member_Attr/, mapping => $mapping );
+        my $group = $self->_parse_ldap_mapping(
+            %args,
+            ldap_entry => $entry,
+            skip => qr/^Member_Attr_Value$/i,
+            mapping => $mapping,
+        );
+        foreach my $key ( grep !/^Member_Attr/, keys %$group ) {
+            @{ $group->{$key} } = map { ref $_ eq 'ARRAY'? $_->[0] : $_ } @{ $group->{$key} };
+            $group->{$key} = join ' ', grep defined && length, @{ $group->{$key} };
+        }
+        @{ $group->{'Member_Attr'} } = map { ref $_ eq 'ARRAY'? @$_ : $_  } @{ $group->{'Member_Attr'} }
+            if $group->{'Member_Attr'};
         $group->{Description} ||= 'Imported from LDAP';
         unless ( $group->{Name} ) {
             $self->_warn("No Name for group, skipping ".Dumper $group);
@@ -979,7 +1154,14 @@ sub _import_group {
     $self->_debug("Processing group $group->{Name}");
     my ($group_obj, $created) = $self->create_rt_group( %args, group => $group );
     return if $args{import} and not $group_obj;
-    $self->add_group_members( %args, name => $group->{Name}, group => $group_obj, ldap_entry => $ldap_entry, new => $created );
+    $self->add_group_members(
+        %args,
+        name => $group->{Name},
+        info => $group,
+        group => $group_obj,
+        ldap_entry => $ldap_entry,
+        new => $created,
+    );
     # XXX TODO: support OCFVs for groups too
     return;
 }
@@ -1004,8 +1186,12 @@ sub create_rt_group {
     my %args = @_;
     my $group = $args{group};
 
-    my $group_obj = RT::Group->new($RT::SystemUser);
-    $group_obj->LoadUserDefinedGroup( $group->{Name} );
+    my $group_obj = $self->find_rt_group(%args);
+    return unless defined $group_obj;
+
+    $group = { map { $_ => $group->{$_} } qw(id Name Description) };
+
+    my $id = delete $group->{'id'};
 
     my $created;
     if ($group_obj->Id) {
@@ -1031,6 +1217,15 @@ sub create_rt_group {
             }
             $created = $val;
             $self->_debug("Created group for $group->{Name} with id ".$group_obj->Id);
+
+            if ( $id ) {
+                my ($val, $msg) = $group_obj->SetAttribute( Name => 'LDAPImport-gid-'.$id, Content => 1 );
+                unless ($val) {
+                    $self->_error("couldn't set attribute: $msg");
+                    return;
+                }
+            }
+
         } else {
             print "Found new group $group->{Name} to create in RT\n";
             $self->_show_group_info( %args );
@@ -1045,6 +1240,102 @@ sub create_rt_group {
 
 }
 
+=head3 find_rt_group
+
+Loads groups by Name and by the specified LDAP id. Attempts to resolve
+renames and other out-of-sync failures between RT and LDAP.
+
+=cut
+
+sub find_rt_group {
+    my $self = shift;
+    my %args = @_;
+    my $group = $args{group};
+
+    my $group_obj = RT::Group->new($RT::SystemUser);
+    $group_obj->LoadUserDefinedGroup( $group->{Name} );
+    return $group_obj unless $group->{'id'};
+
+    unless ( $group_obj->id ) {
+        $self->_debug("No group in RT named $group->{Name}. Looking by $group->{id} LDAP id.");
+        $group_obj = $self->find_rt_group_by_ldap_id( $group->{'id'} );
+        unless ( $group_obj ) {
+            $self->_debug("No group in RT with LDAP id $group->{id}. Creating a new one.");
+            return RT::Group->new($RT::SystemUser);
+        }
+
+        $self->_debug("No group in RT named $group->{Name}, but found group by LDAP id $group->{id}. Renaming the group.");
+        # $group->Update will take care of the name
+        return $group_obj;
+    }
+
+    my $attr_name = 'LDAPImport-gid-'. $group->{'id'};
+    my $rt_gid = $group_obj->FirstAttribute( $attr_name );
+    return $group_obj if $rt_gid;
+
+    my $other_group = $self->find_rt_group_by_ldap_id( $group->{'id'} );
+    if ( $other_group ) {
+        $self->_debug("Group with LDAP id $group->{id} exists, as well as group named $group->{Name}. Renaming both.");
+    }
+    elsif ( grep $_->Name =~ /^LDAPImport-gid-/, @{ $group_obj->Attributes->ItemsArrayRef } ) {
+        $self->_debug("No group in RT with LDAP id $group->{id}, but group $group->{Name} has id. Renaming the group and creating a new one.");
+    }
+    else {
+        $self->_debug("No group in RT with LDAP id $group->{id}, but group $group->{Name} exists and has no LDAP id. Assigning the id to the group.");
+        if ( $args{import} ) {
+            my ($status, $msg) = $group_obj->SetAttribute( Name => $attr_name, Content => 1 );
+            unless ( $status ) {
+                $self->_error("Couldn't set attribute: $msg");
+                return undef;
+            }
+            $self->_debug("Assigned $group->{id} LDAP group id to $group->{Name}");
+        }
+        else {
+            print "Group $group->{'Name'} gets LDAP id $group->{id}\n";
+        }
+
+        return $group_obj;
+    }
+
+    # rename existing group to move it out of our way
+    {
+        my ($old, $new) = ($group_obj->Name, $group_obj->Name .' (LDAPImport '. time . ')');
+        if ( $args{import} ) {
+            my ($status, $msg) = $group_obj->SetName( $new );
+            unless ( $status ) {
+                $self->_error("Couldn't rename group from $old to $new: $msg");
+                return undef;
+            }
+            $self->_debug("Renamed group $old to $new");
+        }
+        else {
+            print "Group $old to be renamed to $new\n";
+        }
+    }
+
+    return $other_group || RT::Group->new($RT::SystemUser);
+}
+
+=head3 find_rt_group_by_ldap_id
+
+Loads an RT::Group by the ldap provided id (different from RT's internal group
+id)
+
+=cut
+
+sub find_rt_group_by_ldap_id {
+    my $self = shift;
+    my $id = shift;
+
+    my $groups = RT::Groups->new( RT->SystemUser );
+    $groups->LimitToUserDefinedGroups;
+    my $attr_alias = $groups->Join( FIELD1 => 'id', TABLE2 => 'Attributes', FIELD2 => 'ObjectId' );
+    $groups->Limit( ALIAS => $attr_alias, FIELD => 'ObjectType', VALUE => 'RT::Group' );
+    $groups->Limit( ALIAS => $attr_alias, FIELD => 'Name', VALUE => 'LDAPImport-gid-'. $id );
+    return $groups->First;
+}
+
+
 =head3 add_group_members
 
 Iterate over the list of values in the C<Member_Attr> LDAP entry.
@@ -1064,8 +1355,7 @@ sub add_group_members {
 
     $self->_debug("Processing group membership for $groupname");
 
-    my $members = $self->_get_group_members_from_ldap(%args);
-
+    my $members = $args{'info'}{'Member_Attr'};
     unless (defined $members) {
         $self->_warn("No members found for $groupname in Member_Attr");
         return;
@@ -1134,17 +1424,6 @@ sub add_group_members {
     }
 }
 
-sub _get_group_members_from_ldap {
-    my $self = shift;
-    my %args = @_;
-    my $ldap_entry = $args{ldap_entry};
-
-    my $mapping = $RT::LDAPGroupMapping;
-
-    my $members = $ldap_entry->get_value($mapping->{Member_Attr}, asref => 1);
-}
-
-
 =head2 _show_group
 
 Show debugging information about the group record we're going to import
@@ -1248,13 +1527,10 @@ sub _warn {
 
 =head1 BUGS AND LIMITATIONS
 
-No bugs have been reported.
-
 Please report any bugs or feature requests to
 C<bug-rt-extension-ldapimport@rt.cpan.org>, or through the web interface at
 L<http://rt.cpan.org>.
 
-
 =head1 AUTHOR
 
 Kevin Falcone  C<< <falcone@bestpractical.com> >>
@@ -1262,7 +1538,7 @@ Kevin Falcone  C<< <falcone@bestpractical.com> >>
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2007-2012, Best Practical Solutions, LLC.  All rights reserved.
+Copyright (c) 2007-2013, Best Practical Solutions, LLC.  All rights reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
